/***************************************************************************
 *   Copyright (C) 2008 by Matthieu Gallien <matthieu_gallien@yahoo.fr>    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/

#include "abstract_task_widget.h"

#include "abstract_tasks_applet.h"
#include "tasks_model.h"

#include <plasma/animator.h>
#include <plasma/theme.h>
#include <plasma/panelsvg.h>
#include <taskmanager/task.h>
#include <taskmanager/taskrmbmenu.h>
#include <KAuthorized>
#include <KColorScheme>
#include <KIconLoader>

#include <QGraphicsItem>
#include <QPainter>
#include <QStyleOptionViewItem>
#include <QContextMenuEvent>
#include <QHoverEvent>
#include <QGraphicsSceneContextMenuEvent>
#include <QTimer>
#include <QTextOption>
#include <QTextLayout>
#include <QApplication>
#include <QGraphicsView>
#include <QGraphicsScene>

bool AbstractTaskWidget::s_backgroundCreated = false;
Plasma::PanelSvg* AbstractTaskWidget::s_taskItemBackground = 0;
qreal AbstractTaskWidget::s_leftMargin = 0;
qreal AbstractTaskWidget::s_topMargin = 0;
qreal AbstractTaskWidget::s_rightMargin = 0;
qreal AbstractTaskWidget::s_bottomMargin = 0;

AbstractTaskWidget::AbstractTaskWidget(AbstractTasksApplet *applet,
                                       const QPersistentModelIndex &index)
    : QGraphicsWidget(applet)
    , m_animId(-1)
    , m_alpha(1)
    , m_fadeIn(true)
    , m_index(index)
    , m_activateTimer(NULL)
    , m_applet(applet)
    , m_mainSize(0.0)
{
    setAcceptDrops(true);
    setAcceptHoverEvents(true);
    setupBackgroundSvg(0);
    connect(Plasma::Theme::defaultTheme(), SIGNAL(themeChanged()), this, SLOT(themeChanged()));
}

void AbstractTaskWidget::setIndex(const QPersistentModelIndex &val)
{
    m_index = val;
    update();
}

const QPersistentModelIndex& AbstractTaskWidget::index() const
{
    return m_index;
}

/** Draws the background for the task item. */
void AbstractTaskWidget::drawBackground(const QRectF &drawZone, QPainter *painter,
                                        const QStyleOptionGraphicsItem *option,
                                        QWidget *widget) const
{
    // FIXME  Check the usage of KColorScheme here with various color schemes

    const QRectF &boundingRect(drawZone);

    //Don't paint with invalid sizes, the happens when the layout i's being initialized
    if (!option->rect.isValid()) {
        return;
    }

    if (!index().isValid()) {
        return;
    }

    const qreal hoverAlpha = 0.4;

    KColorScheme colorScheme(QPalette::Active, KColorScheme::View, Plasma::Theme::defaultTheme()->colorScheme());

    if (index().data(TaskRoles::TaskDemandsAttention).toBool()) {
        if (s_taskItemBackground && s_taskItemBackground->hasElementPrefix("attention")) {
            //Draw task background from theme svg "attention" element
            s_taskItemBackground->setElementPrefix("attention");
            s_taskItemBackground->resizePanel(boundingRect.size());
            s_taskItemBackground->paintPanel(painter, boundingRect);
        } else {
            //Draw task background without svg theming
            QColor background = colorScheme.background(KColorScheme::ActiveBackground).color();
            background.setAlphaF(hoverAlpha + 0.2);
            painter->setBrush(QBrush(background));
            painter->drawPath(Plasma::roundedRectangle(boundingRect, 6));
        }
    } else if (index().data(TaskRoles::IsMinimized).toBool()) {
        //Not painting anything for iconified tasks for now
        painter->setBrush(QBrush());
    } else if (index().data(TaskRoles::IsActive).toBool()) {
            if (s_taskItemBackground && s_taskItemBackground->hasElementPrefix("focus")) {
                //Draw task background from theme svg "focus" element
                s_taskItemBackground->setElementPrefix("focus");
                s_taskItemBackground->resizePanel(boundingRect.size());
                s_taskItemBackground->paintPanel(painter, boundingRect);
            } else {
                //Draw task background without svg theming
                QLinearGradient background(boundingRect.topLeft(),
                                           boundingRect.bottomLeft());
    
                QColor startColor = colorScheme.background(KColorScheme::NormalBackground).color();
                QColor endColor = colorScheme.shade(startColor,KColorScheme::DarkShade);
    
                endColor.setAlphaF(qMin(0.8, startColor.alphaF() + 0.2));
                startColor.setAlphaF(0);
    
                background.setColorAt(0, startColor);
                background.setColorAt(1, endColor);
    
                painter->setBrush(background);
                painter->setPen(Plasma::Theme::defaultTheme()->color(Plasma::Theme::BackgroundColor));

                painter->drawPath(Plasma::roundedRectangle(boundingRect, 6));
            }
    //Default is a normal task
    } else {
        if (s_taskItemBackground && s_taskItemBackground->hasElementPrefix("normal")) {
            //Draw task background from theme svg "normal" element
            s_taskItemBackground->setElementPrefix("normal");
            s_taskItemBackground->resizePanel(boundingRect.size());
            //get the margins now
            s_taskItemBackground->getMargins(s_leftMargin, s_topMargin, s_rightMargin, s_bottomMargin);
            //if the task height is too little reset the top and bottom margins
            if (boundingRect.height() - s_topMargin - s_bottomMargin < KIconLoader::SizeSmall) {
                s_topMargin = 0;
                s_bottomMargin = 0;
            }

            s_taskItemBackground->paintPanel(painter, boundingRect);
        } else {
            //Draw task background without svg theming
            QColor background = colorScheme.shade(colorScheme.background(KColorScheme::AlternateBackground).color(),
                                              KColorScheme::DarkShade);
            background.setAlphaF(0.2);
            painter->setBrush(QBrush(background));
            painter->setPen(Plasma::Theme::defaultTheme()->color(Plasma::Theme::BackgroundColor));

            painter->drawPath(Plasma::roundedRectangle(boundingRect, 6));
        }
    }

    if (option->state & QStyle::State_MouseOver || (m_animId != -1))
    {
        if (s_taskItemBackground && s_taskItemBackground->hasElementPrefix("hover")) {
            //Draw task background from theme svg "hover" element

            s_taskItemBackground->setElementPrefix("hover");
            s_taskItemBackground->resizePanel(boundingRect.size());
            QPixmap pixmap(boundingRect.size().toSize());

            if (option->state & QStyle::State_Sunken) {
                pixmap.fill(QColor(0, 0, 0, 50));
            } else {
                pixmap.fill(QColor(0, 0, 0, 255 * m_alpha));
            }

            QPainter buffPainter(&pixmap);
            buffPainter.setCompositionMode(QPainter::CompositionMode_SourceIn);
            s_taskItemBackground->paintPanel(&buffPainter, boundingRect);
            painter->drawPixmap(boundingRect.topLeft(), pixmap);
        } else {
            //Draw task background without svg theming
            QLinearGradient background(boundingRect.topLeft(),
                                       boundingRect.bottomLeft());

            QColor startColor = colorScheme.background(KColorScheme::AlternateBackground).color();
            QColor endColor = colorScheme.shade(startColor,KColorScheme::DarkShade);

            const qreal pressedAlpha = 0.2;

            qreal startAlpha = 0;

            if (option->state & QStyle::State_Sunken) {
                startAlpha = pressedAlpha;
            } else {
                startAlpha = hoverAlpha;
            }

            startColor.setAlphaF(startAlpha);
            endColor.setAlphaF(m_alpha);

            background.setColorAt(0, startColor);
            background.setColorAt(1, endColor);

            painter->setBrush(background);
            painter->setPen(Plasma::Theme::defaultTheme()->color(Plasma::Theme::BackgroundColor));

            painter->drawPath(Plasma::roundedRectangle(boundingRect, 6));
        }
    }
}

void AbstractTaskWidget::drawTaskIcon(const QRectF &drawZone, QPainter *painter,
                                      const QStyleOptionGraphicsItem *option,
                                      QWidget *widget) const
{
    const QRectF &contentRect(drawZone);

    QPixmap theIcon;

    QSize iconSize(contentRect.width(),
                   contentRect.height());

    if (index().data(TaskRoles::IsTask).toBool()) {
        TaskManager::Task theTask(static_cast<WId>(index().data(TaskRoles::WId).value<qulonglong>()), NULL);

        theIcon = theTask.icon(iconSize.width(), iconSize.height());
    } else {
        theIcon = index().data(TaskRoles::TaskIcon).value<QIcon>().pixmap(iconSize);
    }

    painter->drawPixmap(contentRect.x(), contentRect.y(),
                        iconSize.width(), iconSize.height(), theIcon);
}

QSizeF AbstractTaskWidget::getDesktopNumberSize(const QStyleOptionGraphicsItem *option) const
{
    const QString &desktopNumber(index().data(TaskRoles::DesktopNumber).toString());
    return option->fontMetrics.boundingRect(desktopNumber).size();
}

void AbstractTaskWidget::drawDesktopNumber(const QRectF &drawZone, QPainter *painter,
                                           const QStyleOptionGraphicsItem *option,
                                           QWidget *widget) const
{
    painter->save();
    painter->setBrush(QBrush(QColor(50, 220, 50, 110)));
    painter->setPen(QPen(Qt::NoPen));
    painter->drawEllipse(drawZone);
    painter->restore();

    painter->drawText(drawZone, Qt::AlignCenter,
                      index().data(TaskRoles::DesktopNumber).toString());
}

void AbstractTaskWidget::drawMinimizedFlag(const QRectF &drawZone, QPainter *painter,
                                           const QStyleOptionGraphicsItem *option,
                                           QWidget *widget) const
{
    bool isMinimized(index().data(TaskRoles::IsMinimized).toBool());

    painter->save();
    painter->setBrush(QBrush(QColor(0, 0, 0, 0)));
    painter->setPen(QPen(QColor(50, 220, 50, 200)));

    if (isMinimized) {
        QPolygonF theTriangle(3);
        theTriangle[0] = drawZone.topLeft();
        theTriangle[1] = drawZone.topRight();
        theTriangle[2] = (drawZone.bottomRight() + drawZone.bottomLeft()) / 2;
        painter->drawPolygon(theTriangle);
    } else {
        painter->drawRect(drawZone);
    }
    painter->restore();
}

void AbstractTaskWidget::setupBackgroundSvg(QObject *parent)
{
    if (s_backgroundCreated) {
        return;
    }

    //TODO: we should probably reset this when the theme changes
    s_backgroundCreated = true;
    QString tasksThemePath = Plasma::Theme::defaultTheme()->imagePath("widgets/tasks");
    s_taskItemBackground = 0;

    if (!tasksThemePath.isEmpty()) {
        while (parent && parent->parent()) {
            parent = parent->parent();
        }

        s_taskItemBackground = new Plasma::PanelSvg(parent);
        s_taskItemBackground->setImagePath(tasksThemePath);
        s_taskItemBackground->setCacheAllRenderedPanels(true);
    }
}

void AbstractTaskWidget::contextMenuEvent(QGraphicsSceneContextMenuEvent *event)
{
    if (!KAuthorized::authorizeKAction("kwin_rmb") || index().data(TaskRoles::IsStartup).toBool()) {
        QGraphicsWidget::contextMenuEvent(event);
        return;
    }

    TaskManager::TaskPtr theTask(new TaskManager::Task(static_cast<WId>(index().data(TaskRoles::WId).value<qulonglong>()), NULL));
    TaskManager::TaskRMBMenu menu(theTask);
    menu.exec(event->screenPos());
}

void AbstractTaskWidget::hoverEnterEvent(QGraphicsSceneHoverEvent *event)
{
    const int FadeInDuration = 100;

    if (m_animId != -1) {
        Plasma::Animator::self()->stopCustomAnimation(m_animId);
    }

    m_fadeIn = true;
    m_animId = Plasma::Animator::self()->customAnimation(40 / (1000 / FadeInDuration), FadeInDuration,Plasma::Animator::LinearCurve, this,"animationUpdate");

    QGraphicsWidget::hoverEnterEvent(event);
}

void AbstractTaskWidget::hoverLeaveEvent(QGraphicsSceneHoverEvent *event)
{
    const int FadeOutDuration = 200;

    if (m_animId != -1) {
        Plasma::Animator::self()->stopCustomAnimation(m_animId);
    }

    m_fadeIn = false;
    m_animId = Plasma::Animator::self()->customAnimation(40 / (1000 / FadeOutDuration), FadeOutDuration,Plasma::Animator::LinearCurve, this,"animationUpdate");

    QGraphicsWidget::hoverLeaveEvent(event);
}

void AbstractTaskWidget::mousePressEvent(QGraphicsSceneMouseEvent *event)
{
    update();
}

void AbstractTaskWidget::mouseReleaseEvent(QGraphicsSceneMouseEvent *event)
{
    activate();
}

void AbstractTaskWidget::animationUpdate(qreal progress)
{
    if (progress == 1) {
        m_animId = -1;
        m_fadeIn = true;
    }

    m_alpha = m_fadeIn ? progress : 1 - progress;

    // explicit update
    update();
}

void AbstractTaskWidget::activate()
{
    // the Task class has a method called activateRaiseOrIconify() which
    // should perform the required action here.
    //
    // however it currently does not minimize the task's window if the item
    // is clicked whilst the window is active probably because the active window by
    // the time the mouse is released over the window task button is not the
    // task's window but instead the desktop window
    //
    // TODO: the Kicker panel in KDE 3.x has a feature whereby clicking on it
    // does not take away the focus from the active window (unless clicking
    // in a widget such as a line edit which does accept the focus)
    // this needs to be implemented for Plasma's own panels.
    if (index().data(TaskRoles::IsTask).toBool()) {
        TaskManager::Task theTask(static_cast<WId>(index().data(TaskRoles::WId).value<qulonglong>()), NULL);
        theTask.activateRaiseOrIconify();
        emit windowSelected(this);
    }
}

void AbstractTaskWidget::themeChanged()
{
    setupBackgroundSvg(0);
    QGraphicsWidget::update();
}

void AbstractTaskWidget::dragEnterEvent(QGraphicsSceneDragDropEvent *event)
{
    event->setAccepted(false);
    if (!m_activateTimer) {
        m_activateTimer = new QTimer();
        m_activateTimer->setSingleShot(true);
        m_activateTimer->setInterval(300);
        connect(m_activateTimer, SIGNAL(timeout()), this, SLOT(activate()));
    }
    m_activateTimer->start();
}

void AbstractTaskWidget::dragMoveEvent(QGraphicsSceneDragDropEvent *event)
{
    Q_UNUSED(event);

    // restart the timer so that activate() is only called after the mouse
    // stops moving
    m_activateTimer->start();
}

void AbstractTaskWidget::dragLeaveEvent(QGraphicsSceneDragDropEvent *event)
{
    Q_UNUSED(event);

    delete m_activateTimer;
    m_activateTimer = 0;
}

QTextOption AbstractTaskWidget::textOption() const
{
    Qt::LayoutDirection direction = QApplication::layoutDirection();
    Qt::Alignment alignment = QStyle::visualAlignment(direction, Qt::AlignLeft | Qt::AlignVCenter);

    QTextOption option;
    option.setTextDirection(direction);
    option.setAlignment(alignment);

    return option;
}

QSize AbstractTaskWidget::layoutText(QTextLayout &layout, const QString &text,
                                     const QSize &constraints) const
{
    QFontMetrics metrics(layout.font());
    int leading     = metrics.leading();
    int height      = 0;
    int maxWidth    = constraints.width();
    int widthUsed   = 0;
    int lineSpacing = metrics.lineSpacing();
    QTextLine line;

    layout.setText(text);

    layout.beginLayout();
    while ((line = layout.createLine()).isValid())
    {
        height += leading;

        // Make the last line that will fit infinitely long.
        // drawTextLayout() will handle this by fading the line out
        // if it won't fit in the contraints.
        if (height + 2 * lineSpacing > constraints.height()) {
            line.setPosition(QPoint(0, height));
            break;
        }

        line.setLineWidth(maxWidth);
        line.setPosition(QPoint(0, height));

        height += int(line.height());
        widthUsed = int(qMax(qreal(widthUsed), line.naturalTextWidth()));
    }
    layout.endLayout();

    return QSize(widthUsed, height);
}

void AbstractTaskWidget::drawTextLayout(QPainter *painter, const QTextLayout &layout,
                                        const QRect &rect) const
{
    if (rect.width() < 1 || rect.height() < 1) {
        return;
    }

    QPixmap pixmap(rect.size());
    pixmap.fill(Qt::transparent);

    QPainter p(&pixmap);
    p.setPen(painter->pen());

    // Create the alpha gradient for the fade out effect
    QLinearGradient alphaGradient(0, 0, 1, 0);
    alphaGradient.setCoordinateMode(QGradient::ObjectBoundingMode);
    if (layout.textOption().textDirection() == Qt::LeftToRight)
    {
        alphaGradient.setColorAt(0, QColor(0, 0, 0, 255));
        alphaGradient.setColorAt(1, QColor(0, 0, 0, 0));
    } else
    {
        alphaGradient.setColorAt(0, QColor(0, 0, 0, 0));
        alphaGradient.setColorAt(1, QColor(0, 0, 0, 255));
    }

    QFontMetrics fm(layout.font());
    int textHeight = layout.lineCount() * fm.lineSpacing();
    QPointF position = textHeight < rect.height() ?
            QPointF(0, (rect.height() - textHeight) / 2) : QPointF(0, 0);
    QList<QRect> fadeRects;
    int fadeWidth = 30;

    // Draw each line in the layout
    for (int i = 0; i < layout.lineCount(); i++)
    {
        QTextLine line = layout.lineAt(i);
        line.draw(&p, position);

        // Add a fade out rect to the list if the line is too long
        if (line.naturalTextWidth() > rect.width())
        {
            int x = int(qMin(line.naturalTextWidth(), (qreal)pixmap.width())) - fadeWidth;
            int y = int(line.position().y() + position.y());
            QRect r = QStyle::visualRect(layout.textOption().textDirection(), pixmap.rect(),
                                         QRect(x, y, fadeWidth, int(line.height())));
            fadeRects.append(r);
        }
    }

    // Reduce the alpha in each fade out rect using the alpha gradient
    if (!fadeRects.isEmpty())
    {
        p.setCompositionMode(QPainter::CompositionMode_DestinationIn);
        foreach (const QRect &rect, fadeRects) {
            p.fillRect(rect, alphaGradient);
        }
    }

    p.end();

    painter->drawPixmap(rect.topLeft(), pixmap);
}

AbstractTasksApplet* AbstractTaskWidget::applet() const
{
    return m_applet;
}

void AbstractTaskWidget::publishIconGeometry()
{
    if (!scene()) {
        return;
    }

    QGraphicsView *parentView = 0L;
    // The following was taken from Plasma::Applet, it doesn't make sense to make the item an applet, and this was the easiest way around it.
    foreach (QGraphicsView *view, scene()->views()) {
        if (view->sceneRect().intersects(sceneBoundingRect()) ||
            view->sceneRect().contains(scenePos())) {
            parentView = view;
        }
    }
    if (!parentView || index().data(TaskRoles::IsStartup).toBool()) {
        return;
    }
    if(!boundingRect().isValid()) {
        return;
    }

    QRect rect = parentView->mapFromScene(mapToScene(boundingRect())).boundingRect().adjusted(0, 0, 1, 1);
    rect.moveTopLeft(parentView->mapToGlobal(rect.topLeft()));

    if (index().data(TaskRoles::IsTask).toBool()) {
        TaskManager::Task theTask(static_cast<WId>(index().data(TaskRoles::WId).value<qulonglong>()), NULL);
        theTask.publishIconGeometry(rect);
    }
}

void AbstractTaskWidget::setGeometry(const QRectF& geometry)
{
    QGraphicsWidget::setGeometry(geometry);
    publishIconGeometry();
}

void AbstractTaskWidget::setMainSize(double val)
{
    m_mainSize = val;
    updateGeometry();
}

double AbstractTaskWidget::mainSize() const
{
    return m_mainSize;
}
