/***************************************************************************
 *   Copyright (C) 2008 by Matthieu Gallien <matthieu_gallien@yahoo.fr>    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/

#ifndef ABSTRACTTASKWIDGET_H
#define ABSTRACTTASKWIDGET_H

#include "plasma_tasks_lib_export.h"

#include <QGraphicsWidget>
#include <QPersistentModelIndex>

namespace Plasma
{
    class PanelSvg;
}

class QGraphicsItem;
class QPainter;
class QStyleOptionViewItem;
class QContextMenuEvent;
class QHoverEvent;
class QTimer;
class QTextOption;
class QTextLayout;

class AbstractTasksApplet;

class PLASMA_TASKS_LIB_EXPORT AbstractTaskWidget : public QGraphicsWidget
{

    Q_OBJECT

public:

    explicit AbstractTaskWidget(AbstractTasksApplet *applet,
                                const QPersistentModelIndex &p_index);

    void setupBackgroundSvg(QObject *parent);

    void setIndex(const QPersistentModelIndex &val);

    const QPersistentModelIndex& index() const;

    /** Tells the window manager the minimized task's geometry. */
    void publishIconGeometry();

    /** Overrided from LayoutItem */
    void setGeometry(const QRectF& geometry);

signals:

    /** Emitted when a window is selected for activation, minimization, iconification */
    void windowSelected(AbstractTaskWidget *);

public slots:

    virtual void activate();
    
    virtual void setMainSize(double);

protected:

    AbstractTasksApplet* applet() const;

    void contextMenuEvent(QGraphicsSceneContextMenuEvent *event);
    void hoverEnterEvent(QGraphicsSceneHoverEvent *event);
    void hoverLeaveEvent(QGraphicsSceneHoverEvent *event);
    void mousePressEvent(QGraphicsSceneMouseEvent *event);
    void mouseReleaseEvent(QGraphicsSceneMouseEvent *event);
    void dragEnterEvent(QGraphicsSceneDragDropEvent *event);
    void dragMoveEvent(QGraphicsSceneDragDropEvent *event);
    void dragLeaveEvent(QGraphicsSceneDragDropEvent *event);

    void drawTaskIcon(const QRectF &drawZone, QPainter *painter,
                      const QStyleOptionGraphicsItem *option,
                      QWidget *widget = 0) const;

    QSizeF getDesktopNumberSize(const QStyleOptionGraphicsItem *option) const;

    void drawDesktopNumber(const QRectF &drawZone, QPainter *painter,
                           const QStyleOptionGraphicsItem *option,
                           QWidget *widget = 0) const;

    void drawMinimizedFlag(const QRectF &drawZone, QPainter *painter,
                           const QStyleOptionGraphicsItem *option,
                           QWidget *widget = 0) const;

    /** Draws the background for the task item. */
    void drawBackground(const QRectF &drawZone, QPainter *painter,
                        const QStyleOptionGraphicsItem *option,
                        QWidget *widget = 0) const;

    /** Returns a QTextOption object for the icon label QTtextLayout.*/
    QTextOption textOption() const;

    /**
    * Lays the text out in the text layout using the constraints, and returns the actual
    * size required. The returned size may be wider than the constraints if the text
    * contains a non-breakable word that is wider than the maximum width.
    * If more height is needed than what's available, the last line that will fit will be
    * extended to hold the remainder of the text.
    */
    QSize layoutText(QTextLayout &layout, const QString &text, const QSize &constraints) const;

    /**
    * Draws the text layout (which must already have the text laid out) in the rect using
    * the supplied painter. If the layout contains text lines that are longer than the rect
    * is wide, they will be elided by fading the text out.
    */
    void drawTextLayout(QPainter *painter, const QTextLayout &layout, const QRect &rect) const;
    
    double mainSize() const;

private slots:

    void animationUpdate(qreal progress);

    void themeChanged();

private:

    static bool s_backgroundCreated;
    static Plasma::PanelSvg* s_taskItemBackground;
    static qreal s_leftMargin;
    static qreal s_topMargin;
    static qreal s_rightMargin;
    static qreal s_bottomMargin;

    int m_animId;
    qreal m_alpha;
    bool m_fadeIn;
    
    double m_mainSize;
    
    QPersistentModelIndex m_index;

    QTimer* m_activateTimer;

    AbstractTasksApplet *m_applet;
};

#endif
