/***************************************************************************
 *   Copyright (C) 2008 by Matthieu Gallien <matthieu_gallien@yahoo.fr>    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/

#include "abstract_tasks_applet.h"

#include "tasks_model.h"
#include "tasks_list_view.h"

#include <KDebug>

#include <plasma/dataengine.h>
#include <QPainter>
#include <QGraphicsProxyWidget>
#include <QGraphicsScene>
#include <QGraphicsLinearLayout>

#include <climits>

AbstractTasksApplet::AbstractTasksApplet(QObject* parentObject,
                                         const QVariantList& args)
    : Plasma::Applet(parentObject, args)
    , m_tasksModel(new TasksModel(this))
    , m_list(NULL)
{
}

AbstractTasksApplet::~AbstractTasksApplet()
{
}

void AbstractTasksApplet::taskOrStartupAdded( const QString& newTask )
{
    dataEngine("tasks")->connectSource(newTask, this);
}

void AbstractTasksApplet::taskOrStartupRemoved( const QString& removedTask )
{
    m_tasksModel->removeTaskOrStartup(removedTask);
}

void AbstractTasksApplet::dataUpdated(const QString &sourceName, const Plasma::DataEngine::Data &data)
{
    if (!data.contains("TaskOrStartup")) {
        return;
    }

    if (m_tasksModel->existsTaskOrStartup(sourceName)) {
        m_tasksModel->updateTaskOrStartup(sourceName, data);
    } else {
        m_tasksModel->addTaskOrStartup(sourceName, data);
    }
}

void AbstractTasksApplet::init()
{
    AbstractTaskWidgetFactory *factory = createTaskWidgetFactory();

    m_list = new TasksListView(this);
    m_list->init(factory, m_tasksModel);

    QGraphicsLinearLayout *layout = m_list->getLayout();

    layout->setContentsMargins(0,0,0,0);
    layout->setSizePolicy(QSizePolicy(QSizePolicy::Expanding,QSizePolicy::Expanding));
    //TODO : Qt's bug??
    layout->setMaximumSize(INT_MAX,INT_MAX);

    if (formFactor() == Plasma::Vertical) {
        layout->setOrientation(Qt::Vertical);
    } else {
        layout->setOrientation(Qt::Horizontal);
    }
    setLayout(layout);
    if (formFactor() == Plasma::Vertical) {
        m_list->sizeChanged(contentsRect().width());
        kDebug() << "new size (vertical form factor):" << contentsRect().width();
    } else {
        m_list->sizeChanged(contentsRect().height());
        kDebug() << "new size:" << contentsRect().height();
    }
    updateGeometry();
    
    setAcceptDrops(false);

    connect(dataEngine("tasks"), SIGNAL(sourceAdded(const QString&)),
            this, SLOT(taskOrStartupAdded(const QString&)));

    connect(dataEngine("tasks"), SIGNAL(sourceRemoved(const QString&)),
            this, SLOT(taskOrStartupRemoved(const QString&)));

    const QStringList &existingTasks(dataEngine("tasks")->sources());
    foreach(const QString &task, existingTasks) {
        dataEngine("tasks")->connectSource( task, this );
    }
}

qreal AbstractTasksApplet::itemLeftMargin() const
{
    return m_leftMargin;
}

qreal AbstractTasksApplet::itemRightMargin() const
{
    return m_rightMargin;
}

qreal AbstractTasksApplet::itemTopMargin() const
{
    return m_topMargin;
}

qreal AbstractTasksApplet::itemBottomMargin() const
{
    return m_bottomMargin;
}

void AbstractTasksApplet::constraintsEvent(Plasma::Constraints constraints)
{
    switch(constraints) {
        case Plasma::FormFactorConstraint:
            if (formFactor() == Plasma::Vertical) {
                dynamic_cast<QGraphicsLinearLayout *>(layout())->setOrientation(Qt::Vertical);
            } else {
                dynamic_cast<QGraphicsLinearLayout *>(layout())->setOrientation(Qt::Horizontal);
            }
            break;
        case Plasma::SizeConstraint:
            if (formFactor() == Plasma::Vertical) {
                m_list->sizeChanged(contentsRect().width());
                kDebug() << "new size (vertical form factor):" << contentsRect().width();
            } else {
                m_list->sizeChanged(contentsRect().height());
                kDebug() << "new size:" << contentsRect().height();
            }
            updateGeometry();
            break;
        default:
            break;
    }
    Plasma::Applet::constraintsEvent(constraints);
}
