/***************************************************************************
 *   Copyright (C) 2008 by Matthieu Gallien <matthieu_gallien@yahoo.fr>    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/

#include "tasks_list_view.h"
#include "abstract_task_widget.h"
#include "abstract_task_widget_factory.h"
#include "abstract_tasks_applet.h"

#include <KDebug>

#include <QAbstractItemModel>
#include <QModelIndex>
#include <QGraphicsItem>
#include <QGraphicsLinearLayout>

TasksListView::TasksListView(AbstractTasksApplet *parent)
    : QObject(parent)
    , m_model(NULL)
    , m_layout(NULL)
    , m_parent(parent)
    , m_widget()
    , m_widgetFactory(NULL)
{
    m_layout = new QGraphicsLinearLayout(parent);
}

QGraphicsLinearLayout* TasksListView::getLayout() const
{
    return m_layout;
}

void TasksListView::init(AbstractTaskWidgetFactory *factory, QAbstractItemModel *p_model)
{
    Q_ASSERT(m_widgetFactory == NULL);
    Q_ASSERT(m_model == NULL);
    m_widgetFactory = factory;
    setModel(p_model);
    modelReset();
}

void TasksListView::changeWidgetFactory(AbstractTaskWidgetFactory *factory)
{
    m_widgetFactory = factory;
    modelReset();
}

void TasksListView::dataChanged(const QModelIndex &topLeft, const QModelIndex &bottomRight)
{
    for (int i = topLeft.row(); i <= bottomRight.row(); ++i) {
        m_widget[i]->update();
    }
}

void TasksListView::layoutChanged()
{
    Q_ASSERT(false);
    kDebug() << "hello";
}

void TasksListView::modelReset()
{
    Q_ASSERT(m_widgetFactory != NULL);
    Q_ASSERT(m_model != NULL);
    for (QMap<int, QGraphicsWidget*>::iterator itWidget = m_widget.begin();
         itWidget != m_widget.end(); ++itWidget) {
        m_layout->removeAt(itWidget.key());
        delete itWidget.value();
    }
    m_widget.clear();

    for (int i = 0; i < m_model->rowCount(); ++i) {
        AbstractTaskWidget *theWidget = m_widgetFactory->createTaskWidget(m_parent, m_model->index(i, 0));
        m_widget[i] = theWidget;
        theWidget->setMainSize(m_mainSize);
        m_layout->insertItem(i, theWidget);
    }
}

void TasksListView::rowsInserted(const QModelIndex &parent, int start, int end)
{
    Q_ASSERT(m_widgetFactory != NULL);
    Q_ASSERT(m_model != NULL);
    int oldMax = m_widget.size() - 1;
    if (start <= oldMax) {
        //TODO Debug and test and correct this part
        kDebug() << "moving old widget" << start << oldMax;
        for (int i = start; i <= oldMax; ++i) {
            kDebug() << i << "to" << (end + 1 + i);
            m_widget[end + 1 + i] = m_widget[i];
        }
    }
    for (int i = start; i <= end; ++i) {
        AbstractTaskWidget *theWidget = m_widgetFactory->createTaskWidget(m_parent, m_model->index(i, 0, parent));
        m_widget[i] = theWidget;
        theWidget->setMainSize(m_mainSize);
        m_layout->insertItem(i, theWidget);
    }
}

void TasksListView::rowsRemoved(const QModelIndex &parent, int start, int end)
{
    int oldMax = m_widget.size() - 1;

    for (int i = start; i <= end; ++i) {
        m_layout->removeAt(i);
        delete m_widget[i];
        m_widget.remove(i);
    }

    if (end < oldMax) {
        for (int i = 0; i < oldMax - end; ++i) {
            m_widget[start + i] = m_widget[end + i + 1];
            m_widget.remove(end + i + 1);
        }
    }
}

QAbstractItemModel* TasksListView::model() const
{
    return m_model;
}

void TasksListView::changeModel(QAbstractItemModel *p_model)
{
    setModel(p_model);
}

void TasksListView::setModel(QAbstractItemModel *p_model)
{
    if (model() != NULL) {
        disconnect(model(), SIGNAL(dataChanged(const QModelIndex&, const QModelIndex&)),
                   this, SLOT(dataChanged(const QModelIndex&, const QModelIndex&)));
        disconnect(model(), SIGNAL(layoutChanged()),
                   this, SLOT(layoutChanged()));
        disconnect(model(), SIGNAL(modelReset()),
                   this, SLOT(modelReset()));
        disconnect(model(), SIGNAL(rowsInserted(const QModelIndex&, int, int)),
                   this, SLOT(rowsInserted(const QModelIndex&, int, int)));
        disconnect(model(), SIGNAL(rowsRemoved(const QModelIndex&, int, int)),
                   this, SLOT(rowsRemoved(const QModelIndex&, int, int)));
    }

    m_model = p_model;
    Q_ASSERT(m_model != NULL);

    connect(model(), SIGNAL(dataChanged(const QModelIndex&, const QModelIndex&)),
            this, SLOT(dataChanged(const QModelIndex&, const QModelIndex&)));
    connect(model(), SIGNAL(layoutChanged()),
            this, SLOT(layoutChanged()));
    connect(model(), SIGNAL(modelReset()),
            this, SLOT(modelReset()));
    connect(model(), SIGNAL(rowsInserted(const QModelIndex&, int, int )),
            this, SLOT(rowsInserted(const QModelIndex&, int, int )));
    connect(model(), SIGNAL(rowsRemoved(const QModelIndex&, int, int )),
            this, SLOT(rowsRemoved(const QModelIndex&, int, int )));

    modelReset();
}

void TasksListView::sizeChanged(double val)
{
    m_mainSize = val;
    kDebug() << "new size:" << m_mainSize;
    for (QMap<int, QGraphicsWidget*>::iterator itWidget = m_widget.begin(); itWidget != m_widget.end(); ++itWidget) {
        qobject_cast<AbstractTaskWidget*>(itWidget.value())->setMainSize(m_mainSize);
    }
}
