/***************************************************************************
 *   Copyright (C) 2008 by Matthieu Gallien <matthieu_gallien@yahoo.fr>    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/

#include "tasks_model.h"

#include <KDebug>
#include <KIcon>

TasksModel::TasksModel(QObject *parent)
    : QAbstractListModel(parent)
    , m_tasksOrStartups()
    , m_positionById()
    , m_idByPosition()
{
}

Qt::ItemFlags TasksModel::flags(const QModelIndex& index) const
{
    return Qt::ItemIsEnabled | Qt::ItemIsSelectable;
}

QVariant TasksModel::data(const QModelIndex& index, int role) const
{
    QVariant res;
    if (index.isValid()) {
        const QString &id(idByPosition(index.row()));
        if (isStartup(id)) {
            switch (role) {
            case TaskRoles::TaskName:
                res = taskOrStartupById(id)["text"];
                break;
            case TaskRoles::TaskIcon:
                res = KIcon(taskOrStartupById(id)["icon"].toString());
                break;
            case TaskRoles::DesktopNumber:
                break;
            case TaskRoles::IsMinimized:
                break;
            case TaskRoles::TaskDemandsAttention:
                res = false;
                break;
            case TaskRoles::WId:
                break;
            case TaskRoles::IsActive:
                break;
            case TaskRoles::IsTask:
                res = false;
                break;
            case TaskRoles::IsStartup:
                res = true;
                break;
            default:
                break;
            }
        } else {
            if (isTask(id)) {
                switch (role) {
                  case TaskRoles::TaskName:
                    res = taskOrStartupById(id)["visibleName"];
                    break;
                case TaskRoles::TaskIcon:
                    res = taskOrStartupById(id)["pixmap"];
                    break;
                case TaskRoles::DesktopNumber:
                    res = taskOrStartupById(id)["desktop"];
                    break;
                case TaskRoles::IsMinimized:
                    res = taskOrStartupById(id)["minimized"];
                    break;
                case TaskRoles::TaskDemandsAttention:
                    res = taskOrStartupById(id)["demandsAttention"];
                    break;
                case TaskRoles::WId:
                    res = taskOrStartupById(id)["WId"];
                    break;
                case TaskRoles::IsActive:
                    res = taskOrStartupById(id)["active"];
                    break;
                case TaskRoles::IsTask:
                    res = true;
                    break;
                case TaskRoles::IsStartup:
                    res = false;
                    break;
                default:
                    break;
                }
            }
        }
    }
    return res;
}

int TasksModel::rowCount(const QModelIndex& index) const
{
    return m_positionById.size();
}

QModelIndex TasksModel::indexFromId(const QString& id) const
{
    return createIndex(positionById(id), 0);
}

void TasksModel::addTaskOrStartup(const QString& id, const QHash<QString, QVariant> &taskOrStartup) {
    Q_ASSERT(!existsTaskOrStartup(id));

    int newPosition = rowCount();

    beginInsertRows(QModelIndex(), newPosition, newPosition);

    m_tasksOrStartups[id] = taskOrStartup;
    m_positionById[id] = newPosition;
    m_idByPosition[newPosition] = id;

    endInsertRows();
}

bool TasksModel::removeTaskOrStartup(const QString& id) {
    Q_ASSERT(existsTaskOrStartup(id));

    int oldPosition = positionById(id);

    beginRemoveRows(QModelIndex(), oldPosition, oldPosition);

    const int oldMax = m_positionById.size();

    m_positionById.remove(id);

    for (int i = oldPosition; i < rowCount(); ++i) {
        m_idByPosition[i] = m_idByPosition[i + 1];
        m_positionById[m_idByPosition[i]] = i;
    }

    m_idByPosition.remove(oldMax);

    bool res = m_tasksOrStartups.remove(id) == 1;

    endRemoveRows();

    return res;
}

void TasksModel::updateTaskOrStartup(const QString& id, const QHash<QString, QVariant> &taskOrStartup) {
    Q_ASSERT(existsTaskOrStartup(id));

    m_tasksOrStartups[id] = taskOrStartup;

    emit dataChanged(indexFromId(id), indexFromId(id));
}

bool TasksModel::existsTaskOrStartup(const QString &id) const
{
    return m_tasksOrStartups.contains(id);
}

bool TasksModel::isStartup(const QString &id) const
{
    if (existsTaskOrStartup(id)) {
        return taskOrStartupById(id)["TaskOrStartup"] == "startup";
    } else {
        return false;
    }
}

bool TasksModel::isTask(const QString &id) const
{
    if (existsTaskOrStartup(id)) {
        return taskOrStartupById(id)["TaskOrStartup"] == "task";
    } else {
        return false;
    }
}

int TasksModel::positionById(const QString &id) const
{
    return m_positionById[id];
}

QString TasksModel::idByPosition(int position) const
{
    return m_idByPosition[position];
}

QHash<QString, QVariant> TasksModel::taskOrStartupById(const QString &id) const
{
    return m_tasksOrStartups[id];
}
